/* ScrutariDataURI - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri.tree;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.ThesaurusURI;


/**
 *
 * @author Vincent Calame
 */
public class URITreeBuilder {

    private Map<BaseURI, InternalBaseNode> baseNodeMap = new HashMap<BaseURI, InternalBaseNode>();
    private Map<CorpusURI, InternalCorpusNode> corpusNodeMap = new HashMap<CorpusURI, InternalCorpusNode>();
    private Map<ThesaurusURI, InternalThesaurusNode> thesaurusNodeMap = new HashMap<ThesaurusURI, InternalThesaurusNode>();
    private Map<FicheURI, InternalFicheNode> ficheNodeMap = new HashMap<FicheURI, InternalFicheNode>();
    private Map<MotcleURI, InternalMotcleNode> motcleNodeMap = new HashMap<MotcleURI, InternalMotcleNode>();
    private InternalURITree uriTree = new InternalURITree();

    public URITreeBuilder() {
    }

    public void addScrutariDataURI(ScrutariDataURI scrutariDataURI, Object associatedObject) {
        switch (scrutariDataURI.getType()) {
            case ScrutariDataURI.BASEURI_TYPE:
                add((BaseURI) scrutariDataURI, associatedObject);
                return;
            case ScrutariDataURI.CORPUSURI_TYPE:
                add((CorpusURI) scrutariDataURI, associatedObject);
                return;
            case ScrutariDataURI.THESAURUSURI_TYPE:
                add((ThesaurusURI) scrutariDataURI, associatedObject);
                return;
            case ScrutariDataURI.FICHEURI_TYPE:
                add((FicheURI) scrutariDataURI, associatedObject);
                return;
            case ScrutariDataURI.MOTCLEURI_TYPE:
                add((MotcleURI) scrutariDataURI, associatedObject);
                return;
            default:
                throw new IllegalArgumentException("Unknown type: " + scrutariDataURI.getType());
        }
    }

    public void add(BaseURI baseURI, Object associatedObject) {
        InternalBaseNode baseNode = getBaseNode(baseURI);
        baseNode.setActive(true);
        baseNode.setAssociatedObject(associatedObject);
    }

    public void add(CorpusURI corpusURI, Object associatedObject) {
        InternalCorpusNode corpusNode = getCorpusNode(corpusURI);
        corpusNode.setActive(true);
        corpusNode.setAssociatedObject(associatedObject);
    }

    public void add(ThesaurusURI thesaurusURI, Object associatedObject) {
        InternalThesaurusNode thesaurusNode = getThesaurusNode(thesaurusURI);
        thesaurusNode.setActive(true);
        thesaurusNode.setAssociatedObject(associatedObject);
    }

    public void add(FicheURI ficheURI, Object associatedObject) {
        InternalFicheNode ficheNode = getFicheNode(ficheURI);
        ficheNode.setActive(true);
        ficheNode.setAssociatedObject(associatedObject);
    }

    public void add(MotcleURI motcleURI, Object associatedObject) {
        InternalMotcleNode motcleNode = getMotcleNode(motcleURI);
        motcleNode.setActive(true);
        motcleNode.setAssociatedObject(associatedObject);
    }

    public URITree toURITree() {
        return uriTree;
    }

    private InternalBaseNode getBaseNode(BaseURI baseURI) {
        InternalBaseNode baseNode = baseNodeMap.get(baseURI);
        if (baseNode == null) {
            baseNode = new InternalBaseNode(baseURI);
            baseNodeMap.put(baseURI, baseNode);
            uriTree.addBaseNode(baseNode);
        }
        return baseNode;
    }

    private InternalCorpusNode getCorpusNode(CorpusURI corpusURI) {
        InternalCorpusNode corpusNode = corpusNodeMap.get(corpusURI);
        if (corpusNode == null) {
            corpusNode = new InternalCorpusNode(corpusURI);
            corpusNodeMap.put(corpusURI, corpusNode);
            InternalBaseNode baseNode = getBaseNode(corpusURI.getBaseURI());
            baseNode.addCorpusNode(corpusNode);
        }
        return corpusNode;
    }

    private InternalThesaurusNode getThesaurusNode(ThesaurusURI thesaurusURI) {
        InternalThesaurusNode thesaurusNode = thesaurusNodeMap.get(thesaurusURI);
        if (thesaurusNode == null) {
            thesaurusNode = new InternalThesaurusNode(thesaurusURI);
            thesaurusNodeMap.put(thesaurusURI, thesaurusNode);
            InternalBaseNode baseNode = getBaseNode(thesaurusURI.getBaseURI());
            baseNode.addThesaurusNode(thesaurusNode);
        }
        return thesaurusNode;
    }

    private InternalFicheNode getFicheNode(FicheURI ficheURI) {
        InternalFicheNode ficheNode = ficheNodeMap.get(ficheURI);
        if (ficheNode == null) {
            ficheNode = new InternalFicheNode(ficheURI);
            ficheNodeMap.put(ficheURI, ficheNode);
            InternalCorpusNode corpusNode = getCorpusNode(ficheURI.getCorpusURI());
            corpusNode.addFicheNode(ficheNode);
        }
        return ficheNode;
    }

    private InternalMotcleNode getMotcleNode(MotcleURI motcleURI) {
        InternalMotcleNode motcleNode = motcleNodeMap.get(motcleURI);
        if (motcleNode == null) {
            motcleNode = new InternalMotcleNode(motcleURI);
            motcleNodeMap.put(motcleURI, motcleNode);
            InternalThesaurusNode thesaurusNode = getThesaurusNode(motcleURI.getThesaurusURI());
            thesaurusNode.addMotcleNode(motcleNode);
        }
        return motcleNode;
    }


    private static class InternalURITree implements URITree {

        private List<InternalBaseNode> baseNodeList = new ArrayList<InternalBaseNode>();

        private InternalURITree() {
        }

        public int getBaseCount() {
            return baseNodeList.size();
        }

        public BaseNode getBaseNode(int i) {
            return baseNodeList.get(i);
        }

        private void addBaseNode(InternalBaseNode baseNode) {
            baseNodeList.add(baseNode);
        }

    }


    private static abstract class AbstractURINode implements URINode {

        private boolean active = false;
        private Object associatedObject = null;

        public boolean isActive() {
            return active;
        }

        protected void setActive(boolean active) {
            this.active = active;
        }

        public Object getAssociatedObject() {
            return associatedObject;
        }

        protected void setAssociatedObject(Object associatedObject) {
            this.associatedObject = associatedObject;
        }

    }


    private static class InternalBaseNode extends AbstractURINode implements BaseNode {

        private BaseURI baseURI;
        private List<InternalCorpusNode> corpusNodeList = new ArrayList<InternalCorpusNode>();
        private List<InternalThesaurusNode> thesaurusNodeList = new ArrayList<InternalThesaurusNode>();

        private InternalBaseNode(BaseURI baseURI) {
            this.baseURI = baseURI;
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

        public int getCorpusCount() {
            return corpusNodeList.size();
        }

        public CorpusNode getCorpusNode(int i) {
            return corpusNodeList.get(i);
        }

        public int getThesaurusCount() {
            return thesaurusNodeList.size();
        }

        public ThesaurusNode getThesaurusNode(int i) {
            return thesaurusNodeList.get(i);
        }

        private void addCorpusNode(InternalCorpusNode corpusNode) {
            corpusNodeList.add(corpusNode);
        }

        private void addThesaurusNode(InternalThesaurusNode thesaurusNode) {
            thesaurusNodeList.add(thesaurusNode);
        }

    }


    private static class InternalCorpusNode extends AbstractURINode implements CorpusNode {

        private CorpusURI corpusURI;
        private List<InternalFicheNode> ficheNodeList = new ArrayList<InternalFicheNode>();

        private InternalCorpusNode(CorpusURI corpusURI) {
            this.corpusURI = corpusURI;
        }

        public CorpusURI getCorpusURI() {
            return corpusURI;
        }

        public int getFicheCount() {
            return ficheNodeList.size();
        }

        public FicheNode getFicheNode(int i) {
            return ficheNodeList.get(i);
        }

        private void addFicheNode(InternalFicheNode ficheNode) {
            ficheNodeList.add(ficheNode);
        }

    }


    private static class InternalThesaurusNode extends AbstractURINode implements ThesaurusNode {

        private ThesaurusURI thesaurusURI;
        private List<InternalMotcleNode> motcleNodeList = new ArrayList<InternalMotcleNode>();

        private InternalThesaurusNode(ThesaurusURI thesaurusURI) {
            this.thesaurusURI = thesaurusURI;
        }

        public ThesaurusURI getThesaurusURI() {
            return thesaurusURI;
        }

        public int getMotcleCount() {
            return motcleNodeList.size();
        }

        public MotcleNode getMotcleNode(int i) {
            return motcleNodeList.get(i);
        }

        private void addMotcleNode(InternalMotcleNode motcleNode) {
            motcleNodeList.add(motcleNode);
        }

    }


    private static class InternalFicheNode extends AbstractURINode implements FicheNode {

        private FicheURI ficheURI;

        private InternalFicheNode(FicheURI ficheURI) {
            this.ficheURI = ficheURI;
        }

        public FicheURI getFicheURI() {
            return ficheURI;
        }

    }


    private static class InternalMotcleNode extends AbstractURINode implements MotcleNode {

        private MotcleURI motcleURI;

        private InternalMotcleNode(MotcleURI motcleURI) {
            this.motcleURI = motcleURI;
        }

        public MotcleURI getMotcleURI() {
            return motcleURI;
        }

    }

}
