/* ScrutariLib_Data - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.Label;
import net.scrutari.datauri.MotcleURI;


/**
 *
 * @author Vincent Calame
 */
public final class MotcleData implements Serializable {

    private final MotcleURI motcleURI;
    private final int codemotcle;
    private final int codethesaurus;
    private final int codebase;
    private final Label[] labelArray;

    private MotcleData(MotcleURI motcleURI, int codemotcle, int codethesaurus, int codebase, Label[] labelArray) {
        this.motcleURI = motcleURI;
        this.codemotcle = codemotcle;
        this.codethesaurus = codethesaurus;
        this.codebase = codebase;
        this.labelArray = labelArray;
    }

    public int getCodeMotcle() {
        return codemotcle;
    }

    public MotcleURI getMotcleURI() {
        return motcleURI;
    }

    public int getCodeThesaurus() {
        return codethesaurus;
    }

    public int getCodeBase() {
        return codebase;
    }

    public int getLabelCount() {
        return labelArray.length;
    }

    public Label getLabelByLang(Lang lang) {
        int length = labelArray.length;
        for (int i = 0; i < length; i++) {
            Label label = labelArray[i];
            if (label.getLang().equals(lang)) {
                return label;
            }
        }
        return null;
    }

    public Label getLabelByIndex(int index) {
        return labelArray[index];
    }

    public static MotcleData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        MotcleURI motcleURI = (MotcleURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codemotcle = primitivesReader.readInt();
        int codethesaurus = primitivesReader.readInt();
        int codebase = primitivesReader.readInt();
        int labelLength = primitivesReader.readInt();
        Label[] array = new Label[labelLength];
        for (int i = 0; i < labelLength; i++) {
            Lang lang = Lang.build(primitivesReader.readString());
            String labelString = primitivesReader.readString();
            array[i] = DataLabelBuilder.toLabel(lang, labelString);
        }
        return new MotcleData(motcleURI, codemotcle, codethesaurus, codebase, array);
    }

    public static void toPrimitives(MotcleData motcleData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(motcleData.getMotcleURI(), primitivesWriter);
        primitivesWriter.writeInt(motcleData.codemotcle);
        primitivesWriter.writeInt(motcleData.codethesaurus);
        primitivesWriter.writeInt(motcleData.codebase);
        int length = motcleData.labelArray.length;
        primitivesWriter.writeInt(length);
        for (int i = 0; i < length; i++) {
            Label label = motcleData.labelArray[i];
            primitivesWriter.writeString(label.getLang().toString());
            primitivesWriter.writeString(label.getLabelString());
        }
    }


    public static class Builder {

        private final MotcleURI motcleURI;
        private final List<Label> labelList = new ArrayList<Label>();

        /**
         * Construit une instance de MotcleBuffer pour le mot-clé d'URI
         * <code>motcleURI</code>.
         */
        public Builder(MotcleURI motcleURI) {
            this.motcleURI = motcleURI;
        }

        /**
         * Retourne l'URI du mot-clé.
         */
        public MotcleURI getMotcleURI() {
            return motcleURI;
        }

        /**
         * Ajoute une instance de l'interface Label. S'il existe déjà un objet
         * Label pour la même que le libellé en argument, ce premier libellé est
         * remplacé.
         */
        public void addLabel(Label label) {
            label = DataLabelBuilder.checkLabel(label);
            if (label == null) {
                return;
            }
            labelList.add(label);
        }

        /**
         * Construit une instance de MotcleData à partir des données de l'objet
         * MotcleInfoBuffer.
         */
        public MotcleData toMotcleData(int codemotcle, int codethesaurus, int codebase) {
            int size = labelList.size();
            Label[] labelArray = new Label[size];
            for (int i = 0; i < size; i++) {
                labelArray[i] = labelList.get(i);
            }
            return new MotcleData(motcleURI, codemotcle, codethesaurus, codebase, labelArray);
        }

    }

}
