/* ScrutariLib_SearchEngine - Copyright (c) 2013-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.search;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.ini.IniParser;
import net.scrutari.lexie.AlineaInteger;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;

/**
 *
 * @author Vincent Calame
 */
public final class AlineaFilters {

    public final static AlineaFilter ALL_FILTER = new AllAlineaFilter();
    public final static AlineaFilter TITRE_FILTER = new SpecialAlineaFilter(AlineaInteger.TITRE, "titre");
    public final static AlineaFilter SOUSTITRE_FILTER = new SpecialAlineaFilter(AlineaInteger.SOUSTITRE, "soustitre");
    public final static AlineaFilter MOTCLE_FILTER = new MotcleAlineaFilter("motcle");
    public final static AlineaFilter SUPPLEMENT_FILTER = new SupplementAlineaFilter("supplement");
    private final static Map<String, AlineaFilter> specialFilterMap = new HashMap<String, AlineaFilter>();

    static {
        initSpecialFilterMap(TITRE_FILTER);
        initSpecialFilterMap(SOUSTITRE_FILTER);
        initSpecialFilterMap(MOTCLE_FILTER);
        initSpecialFilterMap(SUPPLEMENT_FILTER);
    }

    private AlineaFilters() {

    }

    public static AlineaFilter getSpecialFilter(String scope) {
        return specialFilterMap.get(scope);
    }

    public static AlineaFilter getAlineaRootCodeFilter(String prefix, int alineaRootCode) {
        return new RootCodeAlineaFilter(prefix, alineaRootCode);
    }

    private static void initSpecialFilterMap(AlineaFilter specialAlineaFilter) {
        String prefix = specialAlineaFilter.getFilterPrefix();
        specialFilterMap.put(prefix, specialAlineaFilter);
        InputStream inputStream = AlineaFilters.class.getResourceAsStream("l10n-" + prefix + ".ini");
        if (inputStream != null) {
            Map<String, String> stringMap = new HashMap<String, String>();
            try {
                IniParser.parseIni(inputStream, stringMap);
                inputStream.close();
            } catch (IOException ioe) {
                throw new InternalResourceException(ioe);
            }
            for (String value : stringMap.values()) {
                specialFilterMap.put(value, specialAlineaFilter);
            }
        }
    }

    private static class AllAlineaFilter implements AlineaFilter {

        private AllAlineaFilter() {

        }

        @Override
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            return true;
        }

        @Override
        public boolean acceptMotcle(int codethesaurus) {
            return true;
        }

        @Override
        public String getFilterPrefix() {
            return "";
        }
    }

    private static class MotcleAlineaFilter implements AlineaFilter {

        private final String prefix;

        private MotcleAlineaFilter(String prefix) {
            this.prefix = prefix;
        }

        @Override
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            return false;
        }

        @Override
        public boolean acceptMotcle(int codethesaurus) {
            return true;
        }

        @Override
        public String getFilterPrefix() {
            return prefix;
        }
    }

    private static class SpecialAlineaFilter implements AlineaFilter {

        private final int specialAlineaInteger;
        private final String prefix;

        private SpecialAlineaFilter(int specialAlineaInteger, String prefix) {
            this.specialAlineaInteger = specialAlineaInteger;
            this.prefix = prefix;
        }

        @Override
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            return (alineaInteger == specialAlineaInteger);
        }

        @Override
        public boolean acceptMotcle(int codethesaurus) {
            return false;
        }

        @Override
        public String getFilterPrefix() {
            return prefix;
        }
    }

    private static class SupplementAlineaFilter implements AlineaFilter {

        private final String prefix;

        private SupplementAlineaFilter(String prefix) {
            this.prefix = prefix;
        }

        @Override
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            short alineaCategory = AlineaInteger.getAlineaCategory(alineaInteger);
            return (alineaCategory != AlineaInteger.SPECIAL_CATEGORY);
        }

        @Override
        public boolean acceptMotcle(int codethesaurus) {
            return false;
        }

        @Override
        public String getFilterPrefix() {
            return prefix;
        }
    }

    private static class RootCodeAlineaFilter implements AlineaFilter {

        private final String prefix;
        private final int filterAlineaRootCode;

        private RootCodeAlineaFilter(String prefix, int filterAlineaRootCode) {
            this.prefix = prefix;
            this.filterAlineaRootCode = filterAlineaRootCode;
        }

        @Override
        public boolean acceptAlinea(int codecorpus, int alineaInteger) {
            int alineaRootCode = AlineaInteger.toAlineaRootCode(alineaInteger);
            return (alineaRootCode == filterAlineaRootCode);
        }

        @Override
        public boolean acceptMotcle(int codethesaurus) {
            return false;
        }

        @Override
        public String getFilterPrefix() {
            return prefix;
        }
    }



}
