/* UtilLib - Copyright (c) 2006-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class ConditionsUtils {

    public final static TextTest EMPTY_TEXTTEST = new InternalTextTest(ConditionsConstants.EMPTY_TEST, "");

    private ConditionsUtils() {
    }

    public static short modeToShort(String s, short defaultMode) {
        if (s == null) {
            return defaultMode;
        }
        if (s.equals("and")) {
            return ConditionsConstants.MODE_AND;
        } else if (s.equals("or")) {
            return ConditionsConstants.MODE_OR;
        }
        return defaultMode;
    }

    public static String modeToString(short mode) {
        switch (mode) {
            case ConditionsConstants.MODE_AND:
                return "and";
            case ConditionsConstants.MODE_OR:
                return "or";
            default:
                throw new IllegalArgumentException();
        }
    }

    public static short getModeInverse(short mode) {
        if (mode == ConditionsConstants.MODE_AND) {
            return ConditionsConstants.MODE_OR;
        }
        return ConditionsConstants.MODE_AND;
    }

    public static boolean accept(TextTest textTest, String contentString) {
        if (contentString == null) {
            contentString = "";
        }
        short testType = textTest.getTestType();
        if (testType == ConditionsConstants.EMPTY_TEST) {
            return (contentString.length() == 0);
        }
        if (testType == ConditionsConstants.NOT_EMPTY_TEST) {
            return (contentString.length() > 0);
        }
        String testString = textTest.getText().toLowerCase();
        contentString = contentString.toLowerCase();
        if (isExcludingPartTestType(testType)) {
            switch (testType) {
                case ConditionsConstants.NOT_MATCHES_TEST:
                    return !contentString.equals(testString);
                case ConditionsConstants.NOT_STARTSWITH_TEST:
                    return !contentString.startsWith(testString);
                case ConditionsConstants.NOT_ENDSWITH_TEST:
                    return !contentString.endsWith(testString);
                default:
                    return !contentString.contains(testString);
            }
        } else {
            switch (testType) {
                case ConditionsConstants.MATCHES_TEST:
                    return contentString.equals(testString);
                case ConditionsConstants.STARTSWITH_TEST:
                    return contentString.startsWith(testString);
                case ConditionsConstants.ENDSWITH_TEST:
                    return contentString.endsWith(testString);
                default:
                    return contentString.contains(testString);
            }
        }
    }

    /**
     * Si la chaine ne contient aucun des caractères suivants : !*; alors les
     * espaces de la chaine sont traités comme des séparateurs
     */
    public static Condition parseSimpleCondition(String conditionString) {
        if (conditionString == null) {
            return null;
        }
        conditionString = conditionString.trim();
        int length = conditionString.length();
        if (length == 0) {
            return null;
        }
        short mode = ConditionsConstants.MODE_AND;
        if (conditionString.charAt(0) == '|') {
            if (length == 1) {
                return null;
            }
            mode = ConditionsConstants.MODE_OR;
            conditionString = conditionString.substring(1).trim();
            length = conditionString.length();
        }
        if ((length == 1) && (conditionString.charAt(0) == '!')) {
            return null;
        }
        for (int i = 0; i < length; i++) {
            char carac = conditionString.charAt(i);
            switch (carac) {
                case '*':
                case ';':
                    return parseCondition(conditionString, mode);
            }
        }
        ConditionBuilder conditionBuilder = new ConditionBuilder(mode);
        String[] tokens = StringUtils.getTokens(conditionString, ' ', StringUtils.EMPTY_EXCLUDE);
        int tokenCount = tokens.length;
        for (int i = 0; i < tokenCount; i++) {
            String token = tokens[i];
            short testType = ConditionsConstants.CONTAINS_TEST;
            if (token.charAt(0) == '!') {
                if (token.length() == 1) {
                    continue;
                }
                testType = ConditionsConstants.NOT_CONTAINS_TEST;
                token = token.substring(1);
            }
            conditionBuilder.addTextTest(toTextTest(testType, token));
        }
        return conditionBuilder.toCondition();
    }

    /**
     * Reconstruit une instance de Condition à partir d'une chaîne. Le
     * point-virgule est le séparateur des différentes conditions.
     */
    public static Condition parseCondition(String conditionString, short mode) {
        ConditionBuilder conditionBuilder = new ConditionBuilder(mode);
        int length = conditionString.length();
        boolean previousEscaped = false;
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < length; i++) {
            char carac = conditionString.charAt(i);
            if (previousEscaped) {
                if (carac != ';') {
                    buf.append('\\');
                }
                buf.append(carac);
                previousEscaped = false;
            } else {
                if (carac == ';') {
                    if (buf.length() > 0) {
                        String testString = buf.toString();
                        TextTest textTest = parseTextTest(testString);
                        if (textTest != null) {
                            conditionBuilder.addTextTest(parseTextTest(testString));
                        }
                        buf = new StringBuilder();
                    }

                } else if (carac == '\\') {
                    previousEscaped = true;
                } else {
                    buf.append(carac);
                }
            }
        }
        if (previousEscaped) {
            buf.append('\\');
        }
        String testString = buf.toString();
        TextTest textTest = parseTextTest(testString);
        if (textTest != null) {
            conditionBuilder.addTextTest(textTest);
        }
        return conditionBuilder.toCondition();
    }

    /**
     * Reconstruit une instance de TextTest à partir d'une chaîne.
     */
    public static TextTest parseTextTest(String testString) {
        if (testString == null) {
            return null;
        }
        testString = testString.trim();
        if (testString.length() == 0) {
            return null;
        }
        boolean non = (testString.charAt(0) == '!');
        if ((non) && (testString.length() == 1)) {
            return EMPTY_TEXTTEST;
        }
        if (non) {
            testString = testString.substring(1);
        }
        String text;
        short testType;
        boolean commence = false;
        if ((testString.charAt(0) == '*') || (testString.charAt(0) == '%')) {
            commence = true;
            text = testString.substring(1);
        } else if (testString.startsWith("..")) {
            commence = true;
            text = testString.substring(2);
        } else {
            text = testString;
        }
        text = StringUtils.cleanString(text);
        int taille = text.length();
        if (text.length() == 0) {
            if (non) {
                testType = ConditionsConstants.EMPTY_TEST;
            } else {
                testType = ConditionsConstants.NOT_EMPTY_TEST;
            }
        } else {
            boolean finit = false;
            if ((text.charAt(taille - 1) == '*') || (text.charAt(taille - 1) == '%')) {
                text = text.substring(0, taille - 1);
                finit = true;
            } else if (text.endsWith("..")) {
                text = text.substring(0, taille - 2);
                finit = true;
            }
            if (commence) {
                if (finit) {
                    if (non) {
                        testType = ConditionsConstants.NOT_CONTAINS_TEST;
                    } else {
                        testType = ConditionsConstants.CONTAINS_TEST;
                    }
                } else {
                    if (non) {
                        testType = ConditionsConstants.NOT_ENDSWITH_TEST;
                    } else {
                        testType = ConditionsConstants.ENDSWITH_TEST;
                    }
                }
            } else {
                if (finit) {
                    if (non) {
                        testType = ConditionsConstants.NOT_STARTSWITH_TEST;
                    } else {
                        testType = ConditionsConstants.STARTSWITH_TEST;
                    }
                } else {
                    if (non) {
                        testType = ConditionsConstants.NOT_MATCHES_TEST;
                    } else {
                        testType = ConditionsConstants.MATCHES_TEST;
                    }
                }
            }
        }
        return toTextTest(testType, text);
    }

    public static TextTest toOpposite(TextTest textTest) {
        short oppositeType = getOppositeTestType(textTest.getTestType());
        return toTextTest(oppositeType, textTest.getText());
    }

    public static short getOppositeTestType(short testType) {
        switch (testType) {
            case ConditionsConstants.STARTSWITH_TEST:
                return ConditionsConstants.NOT_STARTSWITH_TEST;
            case ConditionsConstants.ENDSWITH_TEST:
                return ConditionsConstants.NOT_ENDSWITH_TEST;
            case ConditionsConstants.CONTAINS_TEST:
                return ConditionsConstants.NOT_CONTAINS_TEST;
            case ConditionsConstants.MATCHES_TEST:
                return ConditionsConstants.NOT_MATCHES_TEST;
            case ConditionsConstants.EMPTY_TEST:
                return ConditionsConstants.NOT_EMPTY_TEST;
            case ConditionsConstants.NOT_STARTSWITH_TEST:
                return ConditionsConstants.STARTSWITH_TEST;
            case ConditionsConstants.NOT_ENDSWITH_TEST:
                return ConditionsConstants.ENDSWITH_TEST;
            case ConditionsConstants.NOT_CONTAINS_TEST:
                return ConditionsConstants.CONTAINS_TEST;
            case ConditionsConstants.NOT_MATCHES_TEST:
                return ConditionsConstants.MATCHES_TEST;
            case ConditionsConstants.NOT_EMPTY_TEST:
                return ConditionsConstants.EMPTY_TEST;
            default:
                return -1;
        }
    }

    public static String toSQL(TextTest textTest) {
        StringBuilder buf = new StringBuilder();
        switch (textTest.getTestType()) {
            case ConditionsConstants.EMPTY_TEST:
                buf.append(" IS NULL");
                break;
            case ConditionsConstants.NOT_EMPTY_TEST:
                buf.append(" IS NOT NULL");
                break;
            case ConditionsConstants.STARTSWITH_TEST:
                buf.append(" LIKE '");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case ConditionsConstants.ENDSWITH_TEST:
                buf.append(" LIKE '%");
                preparation(buf, textTest);
                buf.append("'");
                break;
            case ConditionsConstants.CONTAINS_TEST:
                buf.append(" LIKE '%");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case ConditionsConstants.MATCHES_TEST:
                buf.append(" LIKE '");
                preparation(buf, textTest);
                buf.append("'");
                break;
            case ConditionsConstants.NOT_STARTSWITH_TEST:
                buf.append(" NOT LIKE '%");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case ConditionsConstants.NOT_ENDSWITH_TEST:
                buf.append(" NOT LIKE '%");
                preparation(buf, textTest);
                buf.append("'");
                break;
            case ConditionsConstants.NOT_CONTAINS_TEST:
                buf.append(" NOT LIKE '%");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case ConditionsConstants.NOT_MATCHES_TEST:
                buf.append(" NOT LIKE '");
                preparation(buf, textTest);
                buf.append("'");
                break;
        }
        return buf.toString();
    }

    private static void preparation(StringBuilder buf, TextTest textTest) {
        String text = textTest.getText();
        for (int i = 0; i < text.length(); i++) {
            char carac = text.charAt(i);
            if (carac == '\'') {
                buf.append("''");
            } else {
                buf.append(carac);
            }
        }
    }

    public static boolean isExcludingPartTestType(short conditionType) {
        return (conditionType == ConditionsConstants.NOT_STARTSWITH_TEST) || (conditionType == ConditionsConstants.NOT_ENDSWITH_TEST) || (conditionType == ConditionsConstants.NOT_CONTAINS_TEST) || (conditionType == ConditionsConstants.NOT_MATCHES_TEST);
    }

    public static boolean isIncludingPartTestType(short conditionType) {
        return (conditionType == ConditionsConstants.STARTSWITH_TEST) || (conditionType == ConditionsConstants.ENDSWITH_TEST) || (conditionType == ConditionsConstants.CONTAINS_TEST) || (conditionType == ConditionsConstants.MATCHES_TEST);
    }

    public static Condition toPartialCondition(Condition condition) {
        short state = condition.getState();
        if (state == ConditionsConstants.PARTIAL_STATE) {
            return condition;
        }
        if (state == ConditionsConstants.PARTIALOREMPTY_STATE) {
            ConditionBuilder conditionBuilder = new ConditionBuilder(condition.getMode());
            conditionBuilder.setState(ConditionsConstants.PARTIAL_STATE);
            int includingSize = condition.getIncludingTextTestCount();
            int excludingSize = condition.getExcludingTextTestCount();
            for (int i = 0; i < includingSize; i++) {
                conditionBuilder.addTextTest(condition.getIncludingTextTest(i));
            }
            for (int i = 0; i < excludingSize; i++) {
                conditionBuilder.addTextTest(condition.getExcludingTextTest(i));
            }
            return conditionBuilder.toCondition();
        }
        return null;
    }

    public static String conditionToSimpleString(Condition condition) {
        String result = internalConditionToSimpleString(condition);
        if (result == null) {
            return conditionToString(condition);
        } else {
            return result;
        }
    }

    private static String internalConditionToSimpleString(Condition condition) {
        short state = condition.getState();
        int excludingSize = condition.getExcludingTextTestCount();
        if (state == ConditionsConstants.PARTIALOREMPTY_STATE) {
            if (excludingSize == 0) {
                return null;
            }
        } else if (state != ConditionsConstants.PARTIAL_STATE) {
            return null;
        }
        int includingSize = condition.getIncludingTextTestCount();
        StringBuilder buf = new StringBuilder();
        if (condition.getMode() == ConditionsConstants.MODE_OR) {
            buf.append('|');
        }
        for (int i = 0; i < includingSize; i++) {
            TextTest textTest = condition.getIncludingTextTest(i);
            if (textTest.getTestType() != ConditionsConstants.CONTAINS_TEST) {
                return null;
            }
            String text = textTest.getText();
            if (text.indexOf(' ') != -1) {
                return null;
            }
            if (text.equals("!")) {
                return null;
            }
            if (text.equals("|")) {
                return null;
            }
            buf.append(text);
            buf.append(' ');
        }
        for (int i = 0; i < excludingSize; i++) {
            TextTest textTest = condition.getExcludingTextTest(i);
            if (textTest.getTestType() != ConditionsConstants.NOT_CONTAINS_TEST) {
                return null;
            }
            String text = textTest.getText();
            if (text.indexOf(' ') != -1) {
                return null;
            }
            buf.append('!');
            buf.append(text);
            buf.append(' ');
        }
        return buf.toString();
    }

    public static String conditionToString(Condition condition) {
        switch (condition.getState()) {
            case ConditionsConstants.EMPTY_STATE:
                return "!*";
            case ConditionsConstants.NOTEMPTY_STATE:
                return "*";
            case ConditionsConstants.PARTIAL_STATE:
                return textTestListToString(condition);
            case ConditionsConstants.PARTIALOREMPTY_STATE:
                if (condition.getExcludingTextTestCount() > 0) {
                    return textTestListToString(condition);
                } else {
                    return textTestListToString(condition) + ";!*";
                }
            case ConditionsConstants.IMPOSSIBLE_STATE:
                return "*;!*";
            case ConditionsConstants.UNKNOWN_STATE:
            case ConditionsConstants.NEUTRAL_STATE:
                return "";
            default:
                throw new SwitchException("state =" + condition.getState());
        }
    }

    private static String textTestListToString(Condition condition) {
        int includingSize = condition.getIncludingTextTestCount();
        int excludingSize = condition.getExcludingTextTestCount();
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < includingSize; i++) {
            if (buf.length() > 0) {
                buf.append(';');
            }
            buf.append(textTestToString(condition.getIncludingTextTest(i)));
        }
        for (int i = 0; i < excludingSize; i++) {
            if (buf.length() > 0) {
                buf.append(';');
            }
            buf.append(textTestToString(condition.getExcludingTextTest(i)));
        }
        return buf.toString();
    }

    /**
     * Exprime la condition sous forme d'une chaîne.
     */
    public static String textTestToString(TextTest textTest) {
        String text = textTest.getText();
        switch (textTest.getTestType()) {
            case ConditionsConstants.STARTSWITH_TEST:
                return text + "*";
            case ConditionsConstants.ENDSWITH_TEST:
                return "*" + text;
            case ConditionsConstants.CONTAINS_TEST:
                return "*" + text + "*";
            case ConditionsConstants.MATCHES_TEST:
                return text;
            case ConditionsConstants.EMPTY_TEST:
                return "!*";
            case ConditionsConstants.NOT_STARTSWITH_TEST:
                return "!" + text + "*";
            case ConditionsConstants.NOT_ENDSWITH_TEST:
                return "!*" + text;
            case ConditionsConstants.NOT_CONTAINS_TEST:
                return "!*" + text + "*";
            case ConditionsConstants.NOT_MATCHES_TEST:
                return "!" + text;
            case ConditionsConstants.NOT_EMPTY_TEST:
                return "*";
            default:
                return "";
        }
    }

    public static String testTypeToString(short testType) {
        switch (testType) {
            case ConditionsConstants.STARTSWITH_TEST:
                return ".*";
            case ConditionsConstants.ENDSWITH_TEST:
                return "*.";
            case ConditionsConstants.CONTAINS_TEST:
                return "*.*";
            case ConditionsConstants.MATCHES_TEST:
                return ".";
            case ConditionsConstants.EMPTY_TEST:
                return "!*";
            case ConditionsConstants.NOT_STARTSWITH_TEST:
                return "!.*";
            case ConditionsConstants.NOT_ENDSWITH_TEST:
                return "!*.";
            case ConditionsConstants.NOT_CONTAINS_TEST:
                return "!*.*";
            case ConditionsConstants.NOT_MATCHES_TEST:
                return "!.";
            case ConditionsConstants.NOT_EMPTY_TEST:
                return "*";
            default:
                throw new IllegalArgumentException("wrong testType value = " + testType);
        }
    }

    public static short testTypeToShort(String type) {
        if (type.equals(".*")) {
            return ConditionsConstants.STARTSWITH_TEST;
        } else if (type.equals("*.")) {
            return ConditionsConstants.ENDSWITH_TEST;
        } else if (type.equals("*.*")) {
            return ConditionsConstants.CONTAINS_TEST;
        } else if (type.equals(".")) {
            return ConditionsConstants.MATCHES_TEST;
        } else if (type.equals("!*")) {
            return ConditionsConstants.EMPTY_TEST;
        } else if (type.equals("!.*")) {
            return ConditionsConstants.NOT_STARTSWITH_TEST;
        } else if (type.equals("!*.")) {
            return ConditionsConstants.NOT_ENDSWITH_TEST;
        } else if (type.equals("!*.*")) {
            return ConditionsConstants.NOT_CONTAINS_TEST;
        } else if (type.equals("!.")) {
            return ConditionsConstants.NOT_MATCHES_TEST;
        } else if (type.equals("*")) {
            return ConditionsConstants.NOT_EMPTY_TEST;
        } else {
            throw new IllegalArgumentException("wrong type value = " + type);
        }
    }

    /**
     *
     * @throws IllegalArgumentException si testType est inconnu.
     */
    public static short getState(short testType) {
        switch (testType) {
            case ConditionsConstants.STARTSWITH_TEST:
            case ConditionsConstants.ENDSWITH_TEST:
            case ConditionsConstants.CONTAINS_TEST:
            case ConditionsConstants.MATCHES_TEST:
                return ConditionsConstants.PARTIAL_STATE;
            case ConditionsConstants.EMPTY_TEST:
                return ConditionsConstants.EMPTY_STATE;
            case ConditionsConstants.NOT_STARTSWITH_TEST:
            case ConditionsConstants.NOT_ENDSWITH_TEST:
            case ConditionsConstants.NOT_CONTAINS_TEST:
            case ConditionsConstants.NOT_MATCHES_TEST:
                return ConditionsConstants.PARTIALOREMPTY_STATE;
            case ConditionsConstants.NOT_EMPTY_TEST:
                return ConditionsConstants.NOTEMPTY_STATE;
            default:
                throw new IllegalArgumentException("unknown conditionType");
        }
    }

    public static short merge(short mode, short currentState, short addingState) {
        if (currentState == ConditionsConstants.UNKNOWN_STATE) {
            return addingState;
        }
        if (currentState == addingState) {
            return currentState;
        }
        if (mode == ConditionsConstants.MODE_AND) {
            return mergeAndMode(currentState, addingState);
        } else {
            return mergeOrMode(currentState, addingState);
        }
    }

    public static short mergeAndMode(short currentState, short addingState) {
        if (currentState == ConditionsConstants.IMPOSSIBLE_STATE) {
            return ConditionsConstants.IMPOSSIBLE_STATE;
        }
        if (currentState == ConditionsConstants.NEUTRAL_STATE) {
            return addingState;
        }
        if (currentState == ConditionsConstants.EMPTY_STATE) {
            switch (addingState) {
                case ConditionsConstants.NOTEMPTY_STATE:
                case ConditionsConstants.PARTIAL_STATE:
                    return ConditionsConstants.IMPOSSIBLE_STATE;
                case ConditionsConstants.PARTIALOREMPTY_STATE:
                    return ConditionsConstants.EMPTY_STATE;
                default:
                    return currentState;
            }
        }
        if (currentState == ConditionsConstants.NOTEMPTY_STATE) {
            switch (addingState) {
                case ConditionsConstants.EMPTY_STATE:
                    return ConditionsConstants.IMPOSSIBLE_STATE;
                case ConditionsConstants.PARTIAL_STATE:
                case ConditionsConstants.PARTIALOREMPTY_STATE:
                    return ConditionsConstants.PARTIAL_STATE;
                default:
                    return currentState;
            }
        }
        if (currentState == ConditionsConstants.PARTIAL_STATE) {
            switch (addingState) {
                case ConditionsConstants.EMPTY_STATE:
                    return ConditionsConstants.IMPOSSIBLE_STATE;
                case ConditionsConstants.NOTEMPTY_STATE:
                case ConditionsConstants.PARTIALOREMPTY_STATE:
                    return ConditionsConstants.PARTIAL_STATE;
                default:
                    return currentState;
            }
        }
        if (currentState == ConditionsConstants.PARTIALOREMPTY_STATE) {
            switch (addingState) {
                case ConditionsConstants.EMPTY_STATE:
                    return ConditionsConstants.EMPTY_STATE;
                case ConditionsConstants.NOTEMPTY_STATE:
                case ConditionsConstants.PARTIAL_STATE:
                    return ConditionsConstants.PARTIAL_STATE;
                default:
                    return currentState;
            }
        }
        throw new IllegalArgumentException();
    }

    public static short mergeOrMode(short currentState, short addingState) {
        if (currentState == ConditionsConstants.IMPOSSIBLE_STATE) {
            return addingState;
        }
        if (currentState == ConditionsConstants.NEUTRAL_STATE) {
            return ConditionsConstants.NEUTRAL_STATE;
        }
        if (currentState == ConditionsConstants.EMPTY_STATE) {
            switch (addingState) {
                case ConditionsConstants.NOTEMPTY_STATE:
                    return ConditionsConstants.NEUTRAL_STATE;
                case ConditionsConstants.PARTIAL_STATE:
                case ConditionsConstants.PARTIALOREMPTY_STATE:
                    return ConditionsConstants.PARTIALOREMPTY_STATE;
                default:
                    return currentState;
            }
        }
        if (currentState == ConditionsConstants.NOTEMPTY_STATE) {
            switch (addingState) {
                case ConditionsConstants.EMPTY_STATE:
                    return ConditionsConstants.NEUTRAL_STATE;
                case ConditionsConstants.PARTIAL_STATE:
                    return ConditionsConstants.NOTEMPTY_STATE;
                case ConditionsConstants.PARTIALOREMPTY_STATE:
                    return ConditionsConstants.NEUTRAL_STATE;
                default:
                    return currentState;
            }
        }
        if (currentState == ConditionsConstants.PARTIAL_STATE) {
            switch (addingState) {
                case ConditionsConstants.EMPTY_STATE:
                    return ConditionsConstants.PARTIALOREMPTY_STATE;
                case ConditionsConstants.NOTEMPTY_STATE:
                    return ConditionsConstants.NOTEMPTY_STATE;
                case ConditionsConstants.PARTIALOREMPTY_STATE:
                    return ConditionsConstants.PARTIALOREMPTY_STATE;
                default:
                    return currentState;
            }
        }
        if (currentState == ConditionsConstants.PARTIALOREMPTY_STATE) {
            switch (addingState) {
                case ConditionsConstants.EMPTY_STATE:
                    return ConditionsConstants.PARTIALOREMPTY_STATE;
                case ConditionsConstants.NOTEMPTY_STATE:
                    return ConditionsConstants.NEUTRAL_STATE;
                case ConditionsConstants.PARTIAL_STATE:
                    return ConditionsConstants.PARTIALOREMPTY_STATE;
                default:
                    return currentState;
            }
        }
        throw new IllegalArgumentException();
    }

    public static boolean isNeutralState(short state) {
        return ((state == ConditionsConstants.UNKNOWN_STATE) || (state == ConditionsConstants.NEUTRAL_STATE));
    }

    public static boolean isPartialState(short state) {
        return ((state == ConditionsConstants.PARTIAL_STATE) || (state == ConditionsConstants.PARTIALOREMPTY_STATE));
    }

    public static TextTest toTextTest(short testType, String text) {
        return new InternalTextTest(testType, text);
    }

    public static TextTest cloneTextTest(TextTest textTest) {
        return new InternalTextTest(textTest.getTestType(), textTest.getText());
    }


    private static class InternalTextTest implements TextTest {

        private final short testType;
        private final String text;

        public InternalTextTest(short testType, String text) {
            this.testType = testType;
            this.text = text;
        }

        @Override
        public short getTestType() {
            return testType;
        }

        @Override
        public String getText() {
            return text;
        }


    }

}
