/* UtilLib - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import java.text.ParseException;
import java.util.AbstractList;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.RandomAccess;
import java.util.Set;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class LangsUtils {

    public final static Langs EMPTY_LANGS = new EmptyLangs();

    private LangsUtils() {
    }

    public static Langs toCleanLangs(String s) {
        Lang[] array = toCleanLangArray(s);
        return new ArrayLangs(array);
    }

    public static Lang[] toCleanLangArray(String s) {
        String[] tokens = StringUtils.getTechnicalTokens(s, true);
        Set<Lang> langSet = new LinkedHashSet<Lang>();
        for (String token : tokens) {
            try {
                Lang lang = Lang.parse(token);
                langSet.add(lang);
            } catch (ParseException mce) {
            }
        }
        return langSet.toArray(new Lang[langSet.size()]);
    }

    public static String toString(Langs langs, String separator) {
        StringBuilder buf = new StringBuilder();
        int count = langs.getLangCount();
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                buf.append(separator);
            }
            buf.append(langs.getLang(i).toString());
        }
        return buf.toString();
    }

    public static Langs singleton(Lang lang) {
        return new SingletonLangs(lang);
    }

    public static Langs fromArray(Lang[] array, boolean copy) {
        if (copy) {
            int length = array.length;
            Lang[] temp = new Lang[length];
            System.arraycopy(array, 0, temp, 0, length);
            array = temp;
        }
        return new ArrayLangs(array);
    }

    public static Langs fromCollection(Collection<Lang> collection) {
        Lang[] array = collection.toArray(new Lang[collection.size()]);
        return new ArrayLangs(array);
    }

    public static boolean isEqual(Langs langs, Lang[] array) {
        int length = array.length;
        if (length != langs.getLangCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        for (int i = 0; i < length; i++) {
            if (!array[i].equals(langs.getLang(i))) {
                return false;
            }
        }
        return true;
    }

    public static boolean isEqual(Langs langs, Collection<Lang> collection) {
        int length = collection.size();
        if (length != langs.getLangCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        int p = 0;
        for (Lang lang : collection) {
            if (!lang.equals(langs.getLang(p))) {
                return false;
            }
            p++;
        }
        return true;
    }

    public static boolean areEquals(Langs langs1, Langs langs2) {
        int length = langs2.getLangCount();
        if (length != langs1.getLangCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        for (int i = 0; i < length; i++) {
            if (!langs2.getLang(i).equals(langs1.getLang(i))) {
                return false;
            }
        }
        return true;
    }

    public static Lang getNearestLang(Langs langs, Lang lang) {
        int langCount = langs.getLangCount();
        if (langCount == 0) {
            throw new IllegalArgumentException("langList.getLangCount() == 0");
        }
        Lang option1 = null;
        Lang option2 = null;
        Lang rootLang = lang.getRootLang();
        for (int i = 0; i < langCount; i++) {
            Lang testedLang = langs.getLang(i);
            if (testedLang.equals(lang)) {
                return testedLang;
            } else if (testedLang.equals(rootLang)) {
                option1 = testedLang;
            } else if ((testedLang.getRootLang().equals(rootLang))) {
                option2 = testedLang;
            }
        }
        if (option1 != null) {
            return option1;
        }
        if (option2 != null) {
            return option2;
        }
        return langs.getLang(0);
    }

    public static Lang getNearestLang(List<Lang> langList, Lang lang) {
        if (langList.isEmpty()) {
            throw new IllegalArgumentException("langList.isEmpty()");
        }
        Lang option1 = null;
        Lang option2 = null;
        Lang rootLang = lang.getRootLang();
        for (Lang testedLang : langList) {
            if (testedLang.equals(lang)) {
                return testedLang;
            } else if (testedLang.equals(rootLang)) {
                option1 = testedLang;
            } else if ((testedLang.getRootLang().equals(rootLang))) {
                option2 = testedLang;
            }
        }
        if (option1 != null) {
            return option1;
        }
        if (option2 != null) {
            return option2;
        }
        return langList.get(0);
    }

    public static List<Lang> wrap(Lang[] array) {
        return new LangList(array);
    }


    private static class EmptyLangs implements Langs {

        private EmptyLangs() {
        }

        @Override
        public int getLangCount() {
            return 0;
        }

        @Override
        public Lang getLang(int i) {
            throw new IndexOutOfBoundsException("langCount = 0");
        }

    }


    private static class SingletonLangs implements Langs {

        private final Lang lang;

        private SingletonLangs(Lang lang) {
            this.lang = lang;
        }

        @Override
        public int getLangCount() {
            return 1;
        }

        @Override
        public Lang getLang(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("langCount = 1");
            }
            return lang;
        }

    }


    private static class ArrayLangs implements Langs {

        private final Lang[] array;

        private ArrayLangs(Lang[] array) {
            this.array = array;
        }

        @Override
        public int getLangCount() {
            return array.length;
        }

        @Override
        public Lang getLang(int i) {
            return array[i];
        }

    }


    private static class LangList extends AbstractList<Lang> implements RandomAccess {

        private final Lang[] array;

        private LangList(Lang[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public Lang get(int index) {
            return array[index];
        }

    }

}
