/* UtilLib - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.request;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.AbstractList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.RandomAccess;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangPreferenceBuilder;
import net.mapeadores.util.mimetype.MimeTypeConstants;
import net.mapeadores.util.request.xml.RequestConfDOMReader;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public final class RequestUtils {

    public final static List<SupplementaryParameter> EMPTY_SUPPLEMENTARYPARAMETERLIST = Collections.emptyList();
    public final static RequestConf EMPTY_REQUESTCONF = new EmptyRequestConf();

    private RequestUtils() {
    }

    public static AppendableRequestMap merge(RequestMap requestMap, List<SupplementaryParameter> supplementaryParameterList) {
        AppendableRequestMap appendableRequestMap = new AppendableRequestMap(requestMap);
        appendableRequestMap.addSupplementaryParameters(supplementaryParameterList);
        return appendableRequestMap;
    }

    public static RequestConf parseRequestConf(File f) throws IOException {
        Document document = DOMUtils.readDocument(f);
        return parseRequestConf(document);
    }

    public static RequestConf parseRequestConf(Document document) {
        RequestConfBuilder requestConfBuilder = new RequestConfBuilder();
        RequestConfDOMReader reader = new RequestConfDOMReader(requestConfBuilder);
        reader.readConf(document.getDocumentElement());
        return requestConfBuilder.toRequestConf();
    }

    public static short getJsonType(RequestMap requestMap) {
        String jsonType = requestMap.getParameter(RequestConstants.JSONTYPE_PARAMETER);
        if (jsonType == null) {
            if (requestMap.getParameter(RequestConstants.CALLBACK_PARAMETER) != null) {
                return JsonType.JSON_P_TYPE;
            } else {
                return JsonType.JSON_TYPE;
            }
        }
        switch (jsonType) {
            case JsonType.JSON_P_STRING:
                return JsonType.JSON_P_TYPE;
            case JsonType.JSON_IN_HTML_STRING:
                return JsonType.JSON_IN_HTML_TYPE;
            default:
                return JsonType.JSON_TYPE;
        }
    }

    public static boolean isTest(RequestMap requestMap) {
        String testString = requestMap.getParameter(RequestConstants.TEST_PARAMETER);
        if (testString == null) {
            return false;
        }
        testString = testString.toLowerCase();
        if ((testString.equals("1")) || (testString.equals("true"))) {
            return true;
        }
        return false;
    }

    public static boolean isJsonDefined(RequestMap requestMap) {
        OutputInfo outputInfo = OutputInfo.buildFromRequest(requestMap);
        return (outputInfo.getType() == OutputInfo.JSON_TYPE);
    }

    public static String jsonTypeToContentType(short jsonType) {
        switch (jsonType) {
            case JsonType.JSON_TYPE:
                return MimeTypeConstants.JSON;
            case JsonType.JSON_IN_HTML_TYPE:
                return MimeTypeConstants.HTML + ";charset=UTF-8";
            case JsonType.JSON_P_TYPE:
                return MimeTypeConstants.JAVASCRIPT + ";charset=UTF-8";
            default:
                throw new IllegalArgumentException("Unknown type: " + jsonType);
        }
    }

    public static void checkLangPreference(RequestMap requestMap, LangPreferenceBuilder langPreferenceBuilder) {
        String localeParameter = requestMap.getParameter(RequestConstants.LOCALE_PARAMETER);
        if (localeParameter != null) {
            try {
                Lang lang = Lang.parse(localeParameter);
                langPreferenceBuilder.addLang(lang);
            } catch (ParseException pe) {
            }
        }
        Locale[] acceptableLocaleArray = requestMap.getAcceptableLocaleArray();
        if (acceptableLocaleArray != null) {
            int length = acceptableLocaleArray.length;
            for (int i = 0; i < length; i++) {
                try {
                    Lang lang = Lang.fromLocale(acceptableLocaleArray[i]);
                    langPreferenceBuilder.addLang(lang);
                } catch (ParseException pe) {

                }
            }
        }
    }

    public static List<SupplementaryParameter> wrap(SupplementaryParameter[] array) {
        return new SupplementaryParameterList(array);
    }


    private static class EmptyRequestConf implements RequestConf {

        private EmptyRequestConf() {
        }

        @Override
        public List<SupplementaryParameter> getSupplementaryParameterList() {
            return EMPTY_SUPPLEMENTARYPARAMETERLIST;
        }

        @Override
        public AttributeHolder getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
        }

    }


    private static class SupplementaryParameterList extends AbstractList<SupplementaryParameter> implements RandomAccess {

        private final SupplementaryParameter[] array;

        private SupplementaryParameterList(SupplementaryParameter[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public SupplementaryParameter get(int index) {
            return array[index];
        }

    }

}
