/* UtilLib - Copyright (c) 2011-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collections;
import java.util.List;
import net.mapeadores.util.instruction.Argument;
import net.mapeadores.util.instruction.Instruction;
import net.mapeadores.util.instruction.InstructionErrorHandler;
import net.mapeadores.util.instruction.InstructionParser;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.misc.DefBuilder;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public final class LabelUtils {

    public final static ChangeLabelHolder EMPTY_CHANGELABELHOLDER = new EmptyChangeLabelHolder();
    public final static Phrases EMPTY_PHRASES = new EmptyPhrases();


    private LabelUtils() {
    }

    public static boolean areEquals(Label label1, Label label2) {
        if (label1 == null) {
            return (label2 == null);
        } else if (label2 == null) {
            return false;
        }
        if (!label1.getLang().equals(label2.getLang())) {
            return false;
        }
        return label1.getLabelString().equals(label2.getLabelString());
    }

    /*
     * Retourne nul si vide.
     * si l'instruction est incorrecte, retourne un labelHolder vide avec un message d'erreur par défaut.
     * l'instruction doit être sous la forme fr="",en="", etc.
     */
    public static LabelHolder parseInstruction(String s) throws ParseException {
        if (s == null) {
            return null;
        }
        s = s.trim();
        if (s.length() == 0) {
            return null;
        }
        if (s.indexOf('=') == -1) {
            throw new ParseException("#missingEqualCharacter", 0);
        }
        InstructionParseErrorHandler errorHandler = new InstructionParseErrorHandler();
        Instruction instruction = InstructionParser.parse(s, errorHandler);
        if (errorHandler.hasError()) {
            throw new ParseException(errorHandler.getErrorMessage(), 0);
        }
        if (instruction == null) {
            return null;
        }
        ChangeLabelHolderBuilder builder = new ChangeLabelHolderBuilder();
        int length = instruction.getArgumentCount();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            try {
                Lang lang = Lang.parse(argument.getKey());
                builder.putLabel(lang, argument.getValue());
            } catch (ParseException pe) {
            }
        }
        return builder.toChangeLabelHolder();
    }

    /**
     * Retourne nul si vide
     */
    public static Label readLabel(Element element) throws ParseException {
        Lang lang = Lang.parse(element.getAttribute("xml:lang"));
        CleanedString labelString = DOMUtils.contentToCleanedString(element);
        if (labelString == null) {
            return null;
        }
        return toLabel(lang, labelString);
    }

    public static void readLabel(Element element, DefBuilder defBuilder) throws ParseException {
        Label label = readLabel(element);
        if (label != null) {
            defBuilder.putLabel(label);
        }
    }


    public static Label toLabel(Lang lang, CleanedString cleanedString) {
        if (lang == null) {
            throw new IllegalArgumentException("lang is null");
        }
        if (cleanedString == null) {
            throw new IllegalArgumentException("cleanedString is null");
        }
        return new InternalLabel(lang, cleanedString.toString());
    }

    public static String checkLabelString(LabelHolder labelHolder, Lang lang, String defaultString) {
        Label label = getLangPartCheckedLabel(labelHolder, lang);
        if (label != null) {
            return label.getLabelString();
        } else {
            return defaultString;
        }
    }

    public static Label getLangPartCheckedLabel(LabelHolder labelHolder, Lang lang) {
        Label label = labelHolder.getLabelByLang(lang);
        if ((label == null) && (!lang.isRootLang())) {
            label = labelHolder.getLabelByLang(lang.getRootLang());
        }
        return label;
    }

    public static Label getFirstLabel(LabelHolder labelHolder) {
        int count = labelHolder.getLabelCount();
        if (count > 0) {
            return labelHolder.getLabelByIndex(0);
        } else {
            return null;
        }
    }

    public static Label getFirstLabel(LabelHolder labelHolder, Lang[] langArray) {
        for (Lang lang : langArray) {
            Label label = labelHolder.getLabelByLang(lang);
            if (label != null) {
                return label;
            }
        }
        return null;
    }

    public static Label getFirstLabel(LabelHolder labelHolder, Langs langList) {
        int length = langList.getLangCount();
        for (int i = 0; i < length; i++) {
            Lang lang = langList.getLang(i);
            Label label = labelHolder.getLabelByLang(lang);
            if (label != null) {
                return label;
            }
        }
        return null;
    }

    public static void addLabels(XMLWriter xmlWriter, LabelHolder labelHolder) throws IOException {
        addLabels(xmlWriter, labelHolder, null);
    }

    public static void addLabels(XMLWriter xmlWriter, LabelHolder labelHolder, LangFilter langFilter) throws IOException {
        int labelCount = labelHolder.getLabelCount();
        for (int i = 0; i < labelCount; i++) {
            Label label = labelHolder.getLabelByIndex(i);
            if ((langFilter == null) || (langFilter.accept(label.getLang()))) {
                addLabel(xmlWriter, label.getLang(), label.getLabelString());
            }
        }
    }

    public static void addLabel(XMLWriter xmlWriter, Lang lang, String text) throws IOException {
        xmlWriter.startOpenTag("label");
        XMLUtils.addXmlLangAttribute(xmlWriter, lang);
        xmlWriter.endOpenTag();
        xmlWriter.addText(text);
        xmlWriter.closeTag("label", false);
    }


    private static class EmptyChangeLabelHolder implements ChangeLabelHolder {

        private EmptyChangeLabelHolder() {
        }

        @Override
        public int getLabelCount() {
            return 0;
        }

        @Override
        public Label getLabelByIndex(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public Label getLabelByLang(Lang lang) {
            return null;
        }

        @Override
        public int getRemovedLangCount() {
            return 0;
        }

        @Override
        public Lang getRemovedLang(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    public static class InstructionParseErrorHandler implements InstructionErrorHandler {

        private String errorMessage;

        private InstructionParseErrorHandler() {

        }

        @Override
        public void invalidAsciiCharacterError(String part, int row, int col) {
            errorMessage = "#invalidAsciiCharacter: " + part + " (" + row + "," + col + ")#";
        }

        @Override
        public void invalidEndCharacterError(String part, int row, int col) {
            errorMessage = "#invalidEndCharacter: " + part + " (" + row + "," + col + ")#";
        }

        @Override
        public void invalidSeparatorCharacterError(String part, int row, int col) {
            errorMessage = "#invalidSeparatorCharacter: " + part + " (" + row + "," + col + ")#";
        }

        private boolean hasError() {
            return (errorMessage != null);
        }

        private String getErrorMessage() {
            return errorMessage;
        }

    }


    private static class InternalLabel implements Label {

        private final Lang lang;
        private final String labelString;

        private InternalLabel(Lang lang, String labelString) {
            this.lang = lang;
            this.labelString = labelString;
        }

        @Override
        public Lang getLang() {
            return lang;
        }

        @Override
        public String getLabelString() {
            return labelString;
        }

    }


    private static class EmptyPhrases implements Phrases {

        private final List<Phrase> phraseList = Collections.emptyList();

        private EmptyPhrases() {

        }

        @Override
        public List<Phrase> getPhraseList() {
            return phraseList;
        }

        @Override
        public Phrase getPhrase(String name) {
            return null;
        }

    }

}
